#  Created by Gengxin Li and Yuehua Cui on 7/20/2010.
#  Copyright 2010 Michigan State University. All rights reserved.

#  This code is designed for the paper in Li and Cui (2010) Annals of Applied Statistics (in press)
#  "A general statistical framework for dissecting parent-of- origin effects underlying triploid endosperm traits in flowering plants"


data.fn1=function(n,fam,mk,num_ma,dis,dis1,k,mu1,mu2,mu3,sigma_m,sigma_f,sigma_mf,sigma_g,sigma_e)
{
############################################
### estimate recombination fraction        #
############################################

f.r <- function(d) (1-exp(-2*d/100))/2

############################################
###   The prob of QTL given on the Marker 
############################################
fp=function(r1,r2,r)
{
    p1=c((1-r1)*(1-r2)/(1-r), r1*r2/(1-r));
    p2=c((1-r1)*r2/r, r1*(1-r2)/r);
    p3=c(r1*(1-r2)/r, (1-r1)*r2/r);
    p4=c(r1*r2/(1-r), (1-r1)*(1-r2)/(1-r));
    
	p = rbind(p1,p2,p3,p4)
	return(p)
}

f.sphi1=function(M.1,M.2,p)
{
   n=length(M.1)
   z=matrix(0,n,2)
   for (i in 1:n)
   {
   if(M.1[i]==1 && M.2[i]==1) z[i,]=p[1,]
    else if(M.1[i]==1 && M.2[i]==0) z[i,]=p[2,] 
     else if(M.1[i]==0 && M.2[i]==1) z[i,]=p[3,] 
      else if(M.1[i]==0 && M.2[i]==0) z[i,]=p[4,] 
    }
    return(z)
}

f.sphi2=function(M.1,M.2,p)
{
   n=length(M.1)
   z=matrix(0,n,2)
   for (i in 1:n)
   {
   if(M.1[i]==0 && M.2[i]==0) z[i,]=p[1,]
    else if(M.1[i]==0 && M.2[i]==-1) z[i,]=p[2,] 
     else if(M.1[i]==-1 && M.2[i]==0) z[i,]=p[3,] 
      else if(M.1[i]==-1 && M.2[i]==-1) z[i,]=p[4,] 
    }
    return(z)
}

###############################################
# IBD matrix and Additive relationship matrix #
###############################################
f.e=function(sss)
{  e<-matrix(0,nrow=NROW(sss),ncol=NROW(sss))
   for (i in 1:NROW(sss))
   for (j in 1:NROW(sss))
   {if (i==j) {e[i,j]=1}
    if (i!=j) {e[i,j]=0}
   }
  return(e)
}

############################################### family 1
f.mm1=function(z,sss)
{  m<-matrix(0,nrow=NROW(sss),ncol=NROW(sss))
   for (i in 1:NROW(sss))
   for (j in 1:NROW(sss))
   {if (i==j) {m[i,j]=4/3}
    if (i!=j) {m[i,j]=4/3}
   } 
   return(m)
}

f.mffm1=function(z,sss)
{  mffm<-matrix(0,nrow=NROW(sss),ncol=NROW(sss))
   for (i in 1:NROW(sss))
   for (j in 1:NROW(sss))
   {if (i==j) {mffm[i,j]=4/3*z[i,1]}
    if (i!=j) {mffm[i,j]=4/3*z[i,1]*z[j,1]+2/3*z[i,1]*z[j,2]+2/3*z[i,2]*z[j,1]}
   } 
   return(mffm)
}

f.ff1=function(z,sss)
{  f<-matrix(0,nrow=NROW(sss),ncol=NROW(sss))
   for (i in 1:NROW(sss))
   for (j in 1:NROW(sss))
   {if (i==j) {f[i,j]=1/3}
    if (i!=j) {f[i,j]=1/3*z[i,1]*z[j,1]+1/3*z[i,2]*z[j,2]}
   }
   return(f)
}

f.g1=function(sss)
{  g<-matrix(0,nrow=NROW(sss),ncol=NROW(sss))
   for (i in 1:NROW(sss))
   for (j in 1:NROW(sss))
   {if (i==j) {g[i,j]=14/6}
    if (i!=j) {g[i,j]=13/6}
   } 
   return(g)
}

############################################### family 2
f.mm2=function(z,sss)
{  m<-matrix(0,nrow=NROW(sss),ncol=NROW(sss))
   for (i in 1:NROW(sss))
   for (j in 1:NROW(sss))
   {if (i==j) {m[i,j]=4/3}
    if (i!=j) {m[i,j]=4/3*z[i,1]*z[j,1]+4/3*z[i,2]*z[j,2]}
   } 
   return(m)
}

f.mffm2=function(z,sss)
{  mffm<-matrix(0,nrow=NROW(sss),ncol=NROW(sss))
   for (i in 1:NROW(sss))
   for (j in 1:NROW(sss))
   {if (i==j) {mffm[i,j]=4/3*z[i,1]}
    if (i!=j) {mffm[i,j]=4/3*z[i,1]*z[j,1]+2/3*z[i,1]*z[j,2]+2/3*z[i,2]*z[j,1]}
   } 
   return(mffm)
}

f.ff2=function(z,sss)
{  f<-matrix(0,nrow=NROW(sss),ncol=NROW(sss))
   for (i in 1:NROW(sss))
   for (j in 1:NROW(sss))
   {if (i==j) {f[i,j]=1/3}
    if (i!=j) {f[i,j]=1/3}
   }
   return(f)
}

f.g2=function(sss)
{  g<-matrix(0,nrow=NROW(sss),ncol=NROW(sss))
   for (i in 1:NROW(sss))
   for (j in 1:NROW(sss))
   {if (i==j) {g[i,j]=7/3}
    if (i!=j) {g[i,j]=5/3}
   } 
   return(g)
}

############################################### family 3
f.mm3=function(z,sss)
{  m<-matrix(0,nrow=NROW(sss),ncol=NROW(sss))
   for (i in 1:NROW(sss))
   for (j in 1:NROW(sss))
   {if (i==j) {m[i,j]=4/3}
    if (i!=j) {m[i,j]=4/3*z[i,1]*z[j,1]+4/3*z[i,2]*z[j,2]}
   } 
   return(m)
}

f.mffm3=function(z,sss)
{  mffm<-matrix(0,nrow=NROW(sss),ncol=NROW(sss))
   for (i in 1:NROW(sss))
   for (j in 1:NROW(sss))
   {if (i==j) {mffm[i,j]=4/3*z[i,2]}
    if (i!=j) {mffm[i,j]=4/3*z[i,2]*z[j,2]+2/3*z[i,1]*z[j,2]+2/3*z[i,2]*z[j,1]}
   } 
   return(mffm)
}

f.ff3=function(z,sss)
{  f<-matrix(0,nrow=NROW(sss),ncol=NROW(sss))
   for (i in 1:NROW(sss))
   for (j in 1:NROW(sss))
   {if (i==j) {f[i,j]=1/3}
    if (i!=j) {f[i,j]=1/3}
   }
   return(f)
}

f.g3=function(sss)
{  g<-matrix(0,nrow=NROW(sss),ncol=NROW(sss))
   for (i in 1:NROW(sss))
   for (j in 1:NROW(sss))
   {if (i==j) {g[i,j]=7/3}
    if (i!=j) {g[i,j]=5/3}
   } 
   return(g)
}

############################################### family 4
f.mm4=function(z,sss)
{  m<-matrix(0,nrow=NROW(sss),ncol=NROW(sss))
   for (i in 1:NROW(sss))
   for (j in 1:NROW(sss))
   {if (i==j) {m[i,j]=4/3}
    if (i!=j) {m[i,j]=4/3}
   } 
   return(m)
}

f.mffm4=function(z,sss)
{  mffm<-matrix(0,nrow=NROW(sss),ncol=NROW(sss))
   for (i in 1:NROW(sss))
   for (j in 1:NROW(sss))
   {if (i==j) {mffm[i,j]=4/3*z[i,2]}
    if (i!=j) {mffm[i,j]=4/3*z[i,2]*z[j,2]+2/3*z[i,1]*z[j,2]+2/3*z[i,2]*z[j,1]}
   } 
   return(mffm)
}

f.ff4=function(z,sss)
{  f<-matrix(0,nrow=NROW(sss),ncol=NROW(sss))
   for (i in 1:NROW(sss))
   for (j in 1:NROW(sss))
   {if (i==j) {f[i,j]=1/3}
    if (i!=j) {f[i,j]=1/3*z[i,1]*z[j,1]+1/3*z[i,2]*z[j,2]}
   }
   return(f)
}

f.g4=function(sss)
{  g<-matrix(0,nrow=NROW(sss),ncol=NROW(sss))
   for (i in 1:NROW(sss))
   for (j in 1:NROW(sss))
   {if (i==j) {g[i,j]=14/6}
    if (i!=j) {g[i,j]=13/6}
   } 
   return(g)
}

###############################################
# fcn for marker information                  #
###############################################
############################################### family 1  &  family 2
marker1<-function(r,n,ma){
t1=matrix(0,nrow=n,ncol=ma)
for (i in 1:n)
{ x1=runif(ma,0,1)
  if (x1[1]>=1/2) {t1[i,1]=1}
  if (x1[1]<1/2) {t1[i,1]=0}
 
for (j in 2:ma)
{ # family 1
  if (t1[i,j-1]==1 & x1[j]>=r) {t1[i,j]=1}
  if (t1[i,j-1]==1 & x1[j]<r) {t1[i,j]=0}
  if (t1[i,j-1]==0 & x1[j]>=r) {t1[i,j]=0}
  if (t1[i,j-1]==0 & x1[j]<r) {t1[i,j]=1}
}
}
 return (t1)
}

############################################### family 3  &  family 4
marker2<-function(r,n,ma){
t3=matrix(0,nrow=n,ncol=ma)
for (i in 1:n)
{ x3=runif(ma,0,1)

  if (x3[1]>=1/2) {t3[i,1]=-1}
  if (x3[1]<1/2) {t3[i,1]=0}
for (j in 2:ma)
{ 
  if (t3[i,j-1]==-1 & x3[j]>=r) {t3[i,j]=-1}
  if (t3[i,j-1]==-1 & x3[j]<r) {t3[i,j]=0}
  if (t3[i,j-1]==0 & x3[j]>=r) {t3[i,j]=0}
  if (t3[i,j-1]==0 & x3[j]<r) {t3[i,j]=-1}
}
}
 return(t3)
}

###############################################
# fcn for mean and variance of phenotypic     #
###############################################

pheo_sigma<-function(m,f,mffm,g,e,sigma_m,sigma_f,sigma_mf,sigma_g,sigma_e)
 { 
  sigma<-sigma_m*m+sigma_f*f+sigma_mf*mffm+sigma_g*g+sigma_e*e
 return (sigma)
}

pheo_mean<-function(mu,n)
{
 mean1=mu*rep(1,n)
return (mean1)
}

############################################### matrix design
tt=matrix(0,n*fam,(mk+1))
m=matrix(0,n*fam,n*fam)
mffm=matrix(0,n*fam,n*fam)
f=matrix(0,n*fam,n*fam)
g=matrix(0,n*fam,n*fam)
e=matrix(0,n*fam,n*fam)
X=matrix(0,n*fam,num_ma)
n1=fam/4*n; n2=fam/2*n; n3=fam/4*n
 
###############################################
### create marker information                 #
###############################################
ff=1;  r=(1-exp(-2*dis/100))/2

for ( ff in 1:fam)
{ 
  if (ff <= (fam/2)) {tt[(n*ff-n+1):(n*ff),1:mk]=marker1(r,n,mk)} 
                 
  if (ff > (fam/2)) {tt[(n*ff-n+1):(n*ff),1:mk]=marker2(r,n,mk)}
}

###############################################
### QTL location                              #
###############################################
   r1=(1-exp(-2*dis1/100))/2;r2=(1-exp(-2*(dis-dis1)/100))/2
  pp <- fp(r1,r2,r)
  i1=3   
 
 for (i1 in 1:fam)
 {
  if (i1 <= (fam/4) ) {
  ss1=cbind(tt[(n*i1-n+1):(n*i1),k],tt[(n*i1-n+1):(n*i1),k+1])
  Mg1=ss1
  sphi1=f.sphi1(Mg1[,1],Mg1[,2],pp)
  e[(n*i1-n+1):(n*i1),(n*i1-n+1):(n*i1)]=f.e(ss1)
  m[(n*i1-n+1):(n*i1),(n*i1-n+1):(n*i1)]=f.mm1(sphi1,ss1)
  mffm[(n*i1-n+1):(n*i1),(n*i1-n+1):(n*i1)]=f.mffm1(sphi1,ss1)
  f[(n*i1-n+1):(n*i1),(n*i1-n+1):(n*i1)]=f.ff1(sphi1,ss1) 
  g[(n*i1-n+1):(n*i1),(n*i1-n+1):(n*i1)]=f.g1(ss1) }

  if (i1 > (fam/4) & i1 <= (fam/2) ) {
   ss2=cbind(tt[(n*i1-n+1):(n*i1),k],tt[(n*i1-n+1):(n*i1),k+1])
   Mg2=ss2
   sphi2=f.sphi1(Mg2[,1],Mg2[,2],pp)
   e[(n*i1-n+1):(n*i1),(n*i1-n+1):(n*i1)]=f.e(ss2)
   m[(n*i1-n+1):(n*i1),(n*i1-n+1):(n*i1)]=f.mm2(sphi2,ss2)
   mffm[(n*i1-n+1):(n*i1),(n*i1-n+1):(n*i1)]=f.mffm2(sphi2,ss2)
   f[(n*i1-n+1):(n*i1),(n*i1-n+1):(n*i1)]=f.ff2(sphi2,ss2)       
   g[(n*i1-n+1):(n*i1),(n*i1-n+1):(n*i1)]=f.g2(ss2)  }

  if (i1 > (fam/2) & i1 <= (fam/(4/3)) ) {
  ss3=cbind(tt[(n*i1-n+1):(n*i1),k],tt[(n*i1-n+1):(n*i1),k+1])
  Mg3=ss3
  sphi3=f.sphi2(Mg3[,1],Mg3[,2],pp)
  e[(n*i1-n+1):(n*i1),(n*i1-n+1):(n*i1)]=f.e(ss3)
  m[(n*i1-n+1):(n*i1),(n*i1-n+1):(n*i1)]=f.mm3(sphi3,ss3)
  mffm[(n*i1-n+1):(n*i1),(n*i1-n+1):(n*i1)]=f.mffm3(sphi3,ss3)
  f[(n*i1-n+1):(n*i1),(n*i1-n+1):(n*i1)]=f.ff3(sphi3,ss3)   
  g[(n*i1-n+1):(n*i1),(n*i1-n+1):(n*i1)]=f.g3(ss3)  }

  if (i1 > (fam/(4/3)) & i1 <= fam ) {
    ss4=cbind(tt[(n*i1-n+1):(n*i1),k],tt[(n*i1-n+1):(n*i1),k+1])
    Mg4=ss4
    sphi4=f.sphi2(Mg4[,1],Mg4[,2],pp)
    e[(n*i1-n+1):(n*i1),(n*i1-n+1):(n*i1)]=f.e(ss4)
    m[(n*i1-n+1):(n*i1),(n*i1-n+1):(n*i1)]=f.mm4(sphi4,ss4)
    mffm[(n*i1-n+1):(n*i1),(n*i1-n+1):(n*i1)]=f.mffm4(sphi4,ss4)
    f[(n*i1-n+1):(n*i1),(n*i1-n+1):(n*i1)]=f.ff4(sphi4,ss4)
    g[(n*i1-n+1):(n*i1),(n*i1-n+1):(n*i1)]=f.g4(ss4) }
}

############################################### phenotype
  library(MASS)
  mean1=matrix(0,fam*n,1)
  mean1[1:n1,]=pheo_mean(mu1,n1)
  mean1[(n1+1):(n1+n2),]=pheo_mean(mu2,n2)
  mean1[(n1+n2+1):(n1+n2+n3),]=pheo_mean(mu3,n3)

  sigma=pheo_sigma(m,f,mffm,g,e,sigma_m,sigma_f,sigma_mf,sigma_g,sigma_e)
  tt[,mk+1]=as.vector(mvrnorm(1, mean1,sigma))

  return(tt)
}


############################################### parameter description
############################################### n: the number of offspring within each family
############################################### fam: the number of families
############################################### mk: the number of marker
############################################### num_ma: the number of maternal genotype
############################################### dis: the distance between two flanking markers
############################################### dis1: the QTL location in two flanking marker.
############################################### k: the marker that QTL is closely linked to it
############################################### mu: the mean values for each family.
############################################### sigma_m: the additive genetic variance due to the maternal allele expression.
############################################### sigma_m: the additive genetic variance due to the paternal allele expression.
############################################### sigma_m: the additive genetic variance due to the cross sharing.
############################################### sigma_g: the polygene variance.
############################################### sigma_e: the environment variance.
############################################### data.fn1: the number of mean values in the fixed effect is only 3
############################################### data.fn1: the number of maternal genotype varies with the different family.

##################################################
### using the data.fn1 to generate data.txt file #
##################################################

setwd("C:/VC")
n=10;fam=20;mk=6;num_ma=3;
dis=20;dis1=8;k=3;
mu1=8;mu2=6;mu3=4;sigma_m=1;sigma_f=1;sigma_mf=0.5;sigma_g=1;sigma_e=3 
data=data.fn1(n=10,fam=20,mk=6,num_ma=3,dis=20,dis1=8,k=3,mu1=8,mu2=6,mu3=4,sigma_m=1,sigma_f=1,sigma_mf=0.5,sigma_g=1,sigma_e=3)
data
write.table(data,"data.txt")

