#  Created by Gengxin Li and Yuehua Cui on 7/20/2010.
#  Copyright 2010 Michigan State University. All rights reserved.

#  This code is designed for the paper in Li and Cui (2010) Annals of Applied Statistics (in press)
#  "A general statistical framework for dissecting parent-of- origin effects underlying triploid endosperm traits in flowering plants"


############################################## MAIN FUNCTION

 vc.fn=function(n,fam,mk,num_ma,dis,tt)
 {
###############################################
### calculate the recombination fraction      #
###############################################

f.r <- function(d) (1-exp(-2*d/100))/2

###############################################
###   The prob of QTL given on the Marker     #
###############################################
fp=function(r1,r2,r)
{
    p1=c((1-r1)*(1-r2)/(1-r), r1*r2/(1-r));
    p2=c((1-r1)*r2/r, r1*(1-r2)/r);
    p3=c(r1*(1-r2)/r, (1-r1)*r2/r);
    p4=c(r1*r2/(1-r), (1-r1)*(1-r2)/(1-r));
    
	p = rbind(p1,p2,p3,p4)
	return(p)
}

f.sphi1=function(M.1,M.2,p)
{
   n=length(M.1)
   z=matrix(0,n,2)
   for (i in 1:n)
   {
   if(M.1[i]==1 && M.2[i]==1) z[i,]=p[1,]
    else if(M.1[i]==1 && M.2[i]==0) z[i,]=p[2,] 
     else if(M.1[i]==0 && M.2[i]==1) z[i,]=p[3,] 
      else if(M.1[i]==0 && M.2[i]==0) z[i,]=p[4,] 
    }
    return(z)
}

f.sphi2=function(M.1,M.2,p)
{
   n=length(M.1)
   z=matrix(0,n,2)
   for (i in 1:n)
   {
   if(M.1[i]==0 && M.2[i]==0) z[i,]=p[1,]
    else if(M.1[i]==0 && M.2[i]==-1) z[i,]=p[2,] 
     else if(M.1[i]==-1 && M.2[i]==0) z[i,]=p[3,] 
      else if(M.1[i]==-1 && M.2[i]==-1) z[i,]=p[4,] 
    }
    return(z)
}

#################################################
### IBD matrix and Additive relationship matrix #
#################################################

f.e=function(sss)
{  e<-matrix(0,nrow=NROW(sss),ncol=NROW(sss))
   for (i in 1:NROW(sss))
   for (j in 1:NROW(sss))
   {if (i==j) {e[i,j]=1}
    if (i!=j) {e[i,j]=0}
   }
  return(e)
}

############################################### family 1
f.mm1=function(z,sss)
{  m<-matrix(0,nrow=NROW(sss),ncol=NROW(sss))
   for (i in 1:NROW(sss))
   for (j in 1:NROW(sss))
   {if (i==j) {m[i,j]=4/3}
    if (i!=j) {m[i,j]=4/3}
   } 
   return(m)
}

f.mffm1=function(z,sss)
{  mffm<-matrix(0,nrow=NROW(sss),ncol=NROW(sss))
   for (i in 1:NROW(sss))
   for (j in 1:NROW(sss))
   {if (i==j) {mffm[i,j]=4/3*z[i,1]}
    if (i!=j) {mffm[i,j]=4/3*z[i,1]*z[j,1]+2/3*z[i,1]*z[j,2]+2/3*z[i,2]*z[j,1]}
   } 
   return(mffm)
}

f.ff1=function(z,sss)
{  f<-matrix(0,nrow=NROW(sss),ncol=NROW(sss))
   for (i in 1:NROW(sss))
   for (j in 1:NROW(sss))
   {if (i==j) {f[i,j]=1/3}
    if (i!=j) {f[i,j]=1/3*z[i,1]*z[j,1]+1/3*z[i,2]*z[j,2]}
   }
   return(f)
}

f.g1=function(sss)
{  g<-matrix(0,nrow=NROW(sss),ncol=NROW(sss))
   for (i in 1:NROW(sss))
   for (j in 1:NROW(sss))
   {if (i==j) {g[i,j]=14/6}
    if (i!=j) {g[i,j]=13/6}
   } 
   return(g)
}

############################################### family 2
f.mm2=function(z,sss)
{  m<-matrix(0,nrow=NROW(sss),ncol=NROW(sss))
   for (i in 1:NROW(sss))
   for (j in 1:NROW(sss))
   {if (i==j) {m[i,j]=4/3}
    if (i!=j) {m[i,j]=4/3*z[i,1]*z[j,1]+4/3*z[i,2]*z[j,2]}
   } 
   return(m)
}

f.mffm2=function(z,sss)
{  mffm<-matrix(0,nrow=NROW(sss),ncol=NROW(sss))
   for (i in 1:NROW(sss))
   for (j in 1:NROW(sss))
   {if (i==j) {mffm[i,j]=4/3*z[i,1]}
    if (i!=j) {mffm[i,j]=4/3*z[i,1]*z[j,1]+2/3*z[i,1]*z[j,2]+2/3*z[i,2]*z[j,1]}
   } 
   return(mffm)
}

f.ff2=function(z,sss)
{  f<-matrix(0,nrow=NROW(sss),ncol=NROW(sss))
   for (i in 1:NROW(sss))
   for (j in 1:NROW(sss))
   {if (i==j) {f[i,j]=1/3}
    if (i!=j) {f[i,j]=1/3}
   }
   return(f)
}

f.g2=function(sss)
{  g<-matrix(0,nrow=NROW(sss),ncol=NROW(sss))
   for (i in 1:NROW(sss))
   for (j in 1:NROW(sss))
   {if (i==j) {g[i,j]=7/3}
    if (i!=j) {g[i,j]=5/3}
   } 
   return(g)
}

############################################### family 3
f.mm3=function(z,sss)
{  m<-matrix(0,nrow=NROW(sss),ncol=NROW(sss))
   for (i in 1:NROW(sss))
   for (j in 1:NROW(sss))
   {if (i==j) {m[i,j]=4/3}
    if (i!=j) {m[i,j]=4/3*z[i,1]*z[j,1]+4/3*z[i,2]*z[j,2]}
   } 
   return(m)
}

f.mffm3=function(z,sss)
{  mffm<-matrix(0,nrow=NROW(sss),ncol=NROW(sss))
   for (i in 1:NROW(sss))
   for (j in 1:NROW(sss))
   {if (i==j) {mffm[i,j]=4/3*z[i,2]}
    if (i!=j) {mffm[i,j]=4/3*z[i,2]*z[j,2]+2/3*z[i,1]*z[j,2]+2/3*z[i,2]*z[j,1]}
   } 
   return(mffm)
}

f.ff3=function(z,sss)
{  f<-matrix(0,nrow=NROW(sss),ncol=NROW(sss))
   for (i in 1:NROW(sss))
   for (j in 1:NROW(sss))
   {if (i==j) {f[i,j]=1/3}
    if (i!=j) {f[i,j]=1/3}
   }
   return(f)
}

f.g3=function(sss)
{  g<-matrix(0,nrow=NROW(sss),ncol=NROW(sss))
   for (i in 1:NROW(sss))
   for (j in 1:NROW(sss))
   {if (i==j) {g[i,j]=7/3}
    if (i!=j) {g[i,j]=5/3}
   } 
   return(g)
}

############################################### family 4
f.mm4=function(z,sss)
{  m<-matrix(0,nrow=NROW(sss),ncol=NROW(sss))
   for (i in 1:NROW(sss))
   for (j in 1:NROW(sss))
   {if (i==j) {m[i,j]=4/3}
    if (i!=j) {m[i,j]=4/3}
   } 
   return(m)
}

f.mffm4=function(z,sss)
{  mffm<-matrix(0,nrow=NROW(sss),ncol=NROW(sss))
   for (i in 1:NROW(sss))
   for (j in 1:NROW(sss))
   {if (i==j) {mffm[i,j]=4/3*z[i,2]}
    if (i!=j) {mffm[i,j]=4/3*z[i,2]*z[j,2]+2/3*z[i,1]*z[j,2]+2/3*z[i,2]*z[j,1]}
   } 
   return(mffm)
}

f.ff4=function(z,sss)
{  f<-matrix(0,nrow=NROW(sss),ncol=NROW(sss))
   for (i in 1:NROW(sss))
   for (j in 1:NROW(sss))
   {if (i==j) {f[i,j]=1/3}
    if (i!=j) {f[i,j]=1/3*z[i,1]*z[j,1]+1/3*z[i,2]*z[j,2]}
   }
   return(f)
}

f.g4=function(sss)
{  g<-matrix(0,nrow=NROW(sss),ncol=NROW(sss))
   for (i in 1:NROW(sss))
   for (j in 1:NROW(sss))
   {if (i==j) {g[i,j]=14/6}
    if (i!=j) {g[i,j]=13/6}
   } 
   return(g)
}

############################################### fcns
 fn<-function(z){
 u=z[1]; v1=z[2];v2=z[3];v3=z[4];v4=z[5]
 h=exp(u)/(1+exp(u));
 r1=exp(v1)/(exp(v1)+exp(v2)+exp(v3)+exp(v4))
 r2=exp(v2)/(exp(v1)+exp(v2)+exp(v3)+exp(v4))
 r3=exp(v3)/(exp(v1)+exp(v2)+exp(v3)+exp(v4))
 r4=exp(v4)/(exp(v1)+exp(v2)+exp(v3)+exp(v4))
 C<-h*(r1*m+r2*f+r3*mffm+r4*g+(1/h-r1-r2-r3-r4)*e)
 muhat=solve(t(X) %*% solve(C) %*% X)%*% (t(X) %*% solve(C) %*% y)
 sighat<-(t(y-X %*% muhat) %*% solve(C) %*% (y-X %*% muhat))/(length(y))
 (length(y)/2)*log(sighat)+(1/2)*(log(det(C))) 
 }

 fn1<-function(z){
 hg=z[1]
 C<-hg*g+(1-hg)*e
 muhat=solve(t(X) %*% solve(C) %*% X)%*% (t(X) %*% solve(C) %*% y)
 sighat<-(t(y-X %*% muhat) %*% solve(C) %*% (y-X %*% muhat))/(length(y))
 (length(y)/2)*log(sighat)+(1/2)*(log(det(C))) 
 }

 par_mu_sig<-function(z){
 u=z[1]; v1=z[2];v2=z[3];v3=z[4];v4=z[5]
 h=exp(u)/(1+exp(u));
 r1=exp(v1)/(exp(v1)+exp(v2)+exp(v3)+exp(v4))
 r2=exp(v2)/(exp(v1)+exp(v2)+exp(v3)+exp(v4))
 r3=exp(v3)/(exp(v1)+exp(v2)+exp(v3)+exp(v4))
 r4=exp(v4)/(exp(v1)+exp(v2)+exp(v3)+exp(v4))
 C<-h*(r1*m+r2*f+r3*mffm+r4*g+(1/h-r1-r2-r3-r4)*e)
 muhat=solve(t(X) %*% solve(C) %*% X)%*% (t(X) %*% solve(C) %*% y)
 sighat<-(t(y-X %*% muhat) %*% solve(C) %*% (y-X %*% muhat))/(length(y))
 list (muhat=muhat,sighat=sighat)
}

############################################### interval mapping
 m=matrix(0,n*fam,n*fam)
 mffm=matrix(0,n*fam,n*fam)
 f=matrix(0,n*fam,n*fam)
 g=matrix(0,n*fam,n*fam)
 e=matrix(0,n*fam,n*fam)
 X=matrix(0,n*fam,num_ma)

 ddd=seq(0,(mk-1)*dis,by=2)
 len=length(ddd)
 chrpp=c(0,20,40,60,80,100) 
 LR1=matrix(0,len,2); muhat=matrix(0,len,num_ma);par_sig=matrix(0,len,5);ww=1
 
 for (ww in 1:len)
{
      dr.1=ddd[ww]
      nl=length(chrpp)-1

      for (jj in 1:nl)
       {
       if (dr.1 >= chrpp[jj] && dr.1 < chrpp[(jj+1)])  {loc1=jj;k=jj}
       if (dr.1 == chrpp[jj+1] )  {loc1=jj;k=jj}			
       }

    rr=f.r(chrpp[loc1+1]-chrpp[loc1])
    r1=f.r(dr.1-chrpp[loc1])
    r2=f.r(chrpp[(loc1+1)]-dr.1)
    pp <- fp(r1,r2,rr)

############################################### generate IBD matrix
  i3=1   
  for (i3 in 1:fam)
 {
  if (i3 <= (fam/4) ) {
  ss1=cbind(tt[(n*i3-n+1):(n*i3),k],tt[(n*i3-n+1):(n*i3),k+1])
  X[(n*i3-n+1):(n*i3),]=cbind(rep(1,NROW(ss1)),rep(0,NROW(ss1)),rep(0,NROW(ss1)))
  Mg1=ss1
  sphi1=f.sphi1(Mg1[,1],Mg1[,2],pp)
  e[(n*i3-n+1):(n*i3),(n*i3-n+1):(n*i3)]=f.e(ss1)
  m[(n*i3-n+1):(n*i3),(n*i3-n+1):(n*i3)]=f.mm1(sphi1,ss1)
  mffm[(n*i3-n+1):(n*i3),(n*i3-n+1):(n*i3)]=f.mffm1(sphi1,ss1) 
  f[(n*i3-n+1):(n*i3),(n*i3-n+1):(n*i3)]=f.ff1(sphi1,ss1) 
  g[(n*i3-n+1):(n*i3),(n*i3-n+1):(n*i3)]=f.g1(ss1) }

  if (i3 > (fam/4) & i3 <= (fam/2) ) {
   ss2=cbind(tt[(n*i3-n+1):(n*i3),k],tt[(n*i3-n+1):(n*i3),k+1])
   X[(n*i3-n+1):(n*i3),]=cbind(rep(0,NROW(ss2)),rep(1,NROW(ss2)),rep(0,NROW(ss2)))
   Mg2=ss2
   sphi2=f.sphi1(Mg2[,1],Mg2[,2],pp)
   e[(n*i3-n+1):(n*i3),(n*i3-n+1):(n*i3)]=f.e(ss2)
   m[(n*i3-n+1):(n*i3),(n*i3-n+1):(n*i3)]=f.mm2(sphi2,ss2)
   mffm[(n*i3-n+1):(n*i3),(n*i3-n+1):(n*i3)]=f.mffm2(sphi2,ss2)
   f[(n*i3-n+1):(n*i3),(n*i3-n+1):(n*i3)]=f.ff2(sphi2,ss2)       
   g[(n*i3-n+1):(n*i3),(n*i3-n+1):(n*i3)]=f.g2(ss2)  }

  if (i3 > (fam/2) & i3 <= (fam/(4/3)) ) {
  ss3=cbind(tt[(n*i3-n+1):(n*i3),k],tt[(n*i3-n+1):(n*i3),k+1])
  X[(n*i3-n+1):(n*i3),]=cbind(rep(0,NROW(ss3)),rep(1,NROW(ss3)),rep(0,NROW(ss3)))
  Mg3=ss3
  sphi3=f.sphi2(Mg3[,1],Mg3[,2],pp)
  e[(n*i3-n+1):(n*i3),(n*i3-n+1):(n*i3)]=f.e(ss3)
  m[(n*i3-n+1):(n*i3),(n*i3-n+1):(n*i3)]=f.mm3(sphi3,ss3)
  mffm[(n*i3-n+1):(n*i3),(n*i3-n+1):(n*i3)]=f.mffm3(sphi3,ss3)
  f[(n*i3-n+1):(n*i3),(n*i3-n+1):(n*i3)]=f.ff3(sphi3,ss3)   
  g[(n*i3-n+1):(n*i3),(n*i3-n+1):(n*i3)]=f.g3(ss3)  }

  if (i3 > (fam/(4/3)) & i3 <= fam ) {
    ss4=cbind(tt[(n*i3-n+1):(n*i3),k],tt[(n*i3-n+1):(n*i3),k+1])
    X[(n*i3-n+1):(n*i3),]=cbind(rep(0,NROW(ss4)),rep(0,NROW(ss4)),rep(1,NROW(ss4)))
    Mg4=ss4
    sphi4=f.sphi2(Mg4[,1],Mg4[,2],pp)
    e[(n*i3-n+1):(n*i3),(n*i3-n+1):(n*i3)]=f.e(ss4)
    m[(n*i3-n+1):(n*i3),(n*i3-n+1):(n*i3)]=f.mm4(sphi4,ss4)
    mffm[(n*i3-n+1):(n*i3),(n*i3-n+1):(n*i3)]=f.mffm4(sphi4,ss4)
    f[(n*i3-n+1):(n*i3),(n*i3-n+1):(n*i3)]=f.ff4(sphi4,ss4)
    g[(n*i3-n+1):(n*i3),(n*i3-n+1):(n*i3)]=f.g4(ss4)        }
 }

############################################### test  
 X=X; y=tt[,(mk+1)]
 mle0=optimize(fn1,c(0,1),tol=0.0001)
 mle1<-optim(c(0.05,0.05,0.05,0.05,0.05),fn,method="Nelder-Mead")
 par_mu_sig1=par_mu_sig(mle1$par)
 par=mle1$par

 h1=exp(mle1$par[1])/(1+exp(mle1$par[1]));
 r11=exp(mle1$par[2])/(exp(mle1$par[2])+exp(mle1$par[3])+exp(mle1$par[4])+exp(mle1$par[5]))
 r22=exp(mle1$par[3])/(exp(mle1$par[2])+exp(mle1$par[3])+exp(mle1$par[4])+exp(mle1$par[5]))
 r33=exp(mle1$par[4])/(exp(mle1$par[2])+exp(mle1$par[3])+exp(mle1$par[4])+exp(mle1$par[5]))
 r44=exp(mle1$par[5])/(exp(mle1$par[2])+exp(mle1$par[3])+exp(mle1$par[4])+exp(mle1$par[5]))

 h11=r11*h1;h22=r22*h1;h33=r33*h1;h44=r44*h1;h55=(1-h11-h22-h33-h44);
 par1=c(h11*par_mu_sig1$sighat,h22*par_mu_sig1$sighat,h33*par_mu_sig1$sighat,h44*par_mu_sig1$sighat,h55*par_mu_sig1$sighat)
 
 LR1[ww,]=c(ddd[ww],2*(mle0$objective-mle1$value))
 muhat[ww,]=t(par_mu_sig1$muhat)
 par_sig[ww,]=par1
}
 colnames(LR1)=c("marker","LR")
 colnames(muhat)=c("mu1","mu2","mu3")
 colnames(par_sig)=c("sigma_m","sigma_f","sigma_mf","sigma_g","sigma_e")
 LR2=LR1
 LR2=as.data.frame(LR2)
 plot(LR~marker, type="l", col="blue",lwd=2,data=LR2)
 list(LR1=LR1,muhat=muhat,par_sig=par_sig)
}

############################################### 
### Implementation                            #
###############################################
setwd("C:/VC")
n=10;fam=20;mk=6;num_ma=3;dis=20
data=read.table("data.txt")
main_result=vc.fn(n=10,fam=20,mk=6,num_ma=3,dis=20,tt=data)
main_result


############################################### 
### Result1: get the likelihood ratio(LR)     #    
###############################################
main_result$LR1[max(main_result$LR1[,2])]
#write.table(main_result$LR1,"LR.txt")


############################################### 
### Result2: get the fixed effect             #    
###############################################
main_result$muhat
#write.table(main_result$muhat,"muhat.txt")

############################################### 
### Result3: get the random effect            #    
###############################################
main_result$par_sig
#write.table(main_result$par_sig,"par_sig.txt")


